
#include "pascalstring.h"

namespace optimized_string
{

PascalString::PascalString()
:	m_usLength(0),
	m_szString(NULL),
	m_usBufferLength(0)
{
}

PascalString::PascalString(const PascalString& strOther)
:	m_usLength(0),
	m_szString(NULL),
	m_usBufferLength(0)
{
	this->Set(strOther);		
}

PascalString::PascalString(const char* szString, const size_t& Length)
:	m_usLength(0),
	m_szString(NULL),
	m_usBufferLength(0)
{
	const unsigned short usLength = static_cast<const unsigned short>(Length);
	this->Set(szString, usLength);
}

PascalString::~PascalString()
{
	if (this->m_szString)
	{
		delete[] this->m_szString;
	}
}

void PascalString::Reset()
{
	if (this->m_usLength > 0)
	{
		if (this->m_szString)
		{
			delete[] this->m_szString;
			this->m_szString = 0;
		}

		this->m_usLength = 0;
	}
}

void PascalString::Set(const PascalString& Other)
{
	this->Reset();
	this->_Allocate(Other.m_usLength);

	this->m_usLength = Other.m_usLength;
	memcpy(this->m_szString, 
		   &Other.m_szString, 
		   sizeof(char) * this->m_usLength);
}

void PascalString::Set(const char* szString, const unsigned short usLength)
{
	this->Reset();
	this->_Allocate(usLength);

	this->m_usLength = usLength;
	memcpy(this->m_szString, szString, sizeof(char) * this->m_usLength);
}

void PascalString::Append(const PascalString& Other, const size_t& Length)
{
	unsigned short usLength = static_cast<unsigned short>(Length);
	this->_CheckGrow(usLength);

	// append
	memcpy(&this->m_szString[this->m_usLength], 
		   Other.m_szString, 
		   sizeof(char) * usLength);
	this->m_usLength += usLength;
}

void PascalString::Append(const char* szString, const size_t& Length)
{
	unsigned short usLength = static_cast<unsigned short>(Length);
	this->_CheckGrow(usLength);

	// append
	memcpy(&this->m_szString[this->m_usLength], 
		   szString, 
		   sizeof(char) * usLength);
	this->m_usLength += usLength;
}

const bool PascalString::Compare(const PascalString& Other) const
{
	if (this->m_usLength == Other.m_usLength)
	{
		if (memcmp(this->m_szString, Other.m_szString, this->m_usLength) == 0)
			return true;
	}

	return false;
}

const bool PascalString::Compare(const char* szString, const size_t& Length) const
{
	unsigned short usLength = static_cast<unsigned short>(Length);
	if (usLength == this->m_usLength)
	{
		if (memcmp(this->m_szString, szString, Length) == 0)
			return true;
	}

	return false;
}

void PascalString::LTrim()
{
	unsigned short usNumToMove = 0;
	while (this->m_szString[usNumToMove] == ' ' ||
		   this->m_szString[usNumToMove] == '\t' ||
		   this->m_szString[usNumToMove] == '\r' ||
		   this->m_szString[usNumToMove] == '\n' ||
		   this->m_szString[usNumToMove] == '\0')
	{
		usNumToMove++;
	}

	this->m_usLength -= usNumToMove;
	memmove(this->m_szString, 
		    &this->m_szString[usNumToMove], 
			sizeof(char) * this->m_usLength);
}

void PascalString::RTrim()
{
	unsigned short usCurrentPos = this->m_usLength - 1;
	while (this->m_szString[usCurrentPos] == ' ' ||
		   this->m_szString[usCurrentPos] == '\t' ||
		   this->m_szString[usCurrentPos] == '\r' ||
		   this->m_szString[usCurrentPos] == '\n' ||
		   this->m_szString[usCurrentPos] == '\0')
	{
		usCurrentPos--;
	}

	this->m_usLength = usCurrentPos + 1;
}

size_t PascalString::Len()
{
	return this->m_usLength;
}

const size_t PascalString::Len() const
{
	return this->m_usLength;
}

char* PascalString::GetString() 
{
	return this->m_szString;
}

const char* PascalString::GetString() const
{
	return this->m_szString;
}

const char* PascalString::GetCString() 
{
	this->Append("\0", 1);
	return this->m_szString;
}

void PascalString::_Allocate(unsigned short usLength)
{
	unsigned short usAllocSize = DEFAULT_ALLOC_SIZE;
	if (usLength > DEFAULT_ALLOC_SIZE)
	{
		usAllocSize = usLength;
	}

	this->m_szString = new char[usAllocSize];
	this->m_usBufferLength = usAllocSize;
}

void PascalString::_CheckGrow(unsigned short usLength)
{
	if ((this->m_usLength + usLength) > this->m_usBufferLength)
	{
		unsigned short usSizeDiff;
		usSizeDiff = (this->m_usLength + usLength) - this->m_usBufferLength;
		unsigned short usGrowSize = DEFAULT_GROW_SIZE;

		if (usSizeDiff > DEFAULT_GROW_SIZE)
		{
			usGrowSize = usSizeDiff;
		}

		if (this->m_usBufferLength >= 60000)
			this->m_usBufferLength = this->m_usBufferLength;

		unsigned int uiNewSize = this->m_usBufferLength + usGrowSize;
		unsigned short usNewSize = static_cast<unsigned short>(uiNewSize);
		if (uiNewSize > PascalString::MAX_LENGTH)
			usNewSize = PascalString::MAX_LENGTH;

		char* szNew = new char[usNewSize];
		memcpy(szNew, this->m_szString, sizeof(char) * this->m_usLength);
		delete[] this->m_szString;
		this->m_szString = szNew;
		this->m_usBufferLength = usNewSize;
	}
}

};