#ifndef STRING_H
#define STRING_H

#include <string.h>
#include "pascalstring.h"
#include "cstring.h"

namespace optimized_string
{

class String
{
public:
	enum TYPE
	{
		PASCAL_STRING,
		C_STRING,
	};

public:
	// constructor / destructor
	String();
	String(const String& Other);
	String (const char* szString);
	~String();

	// set / reset
	void Reset();
	void Set(const String& Other);
	void Set(const char* szString);

	// basic string functionality
	void Append(const String& Other);
	void Append(const char* szString);
	const bool Compare(const String& Other) const;
	const bool Compare(const char* szString) const;
	void LTrim();
	void RTrim();
	size_t Len();
	const size_t Len() const;
	const char* GetCString();

	// operators
	inline String& operator = (const String& Other) 
	{ 
		this->Set(Other); return (*this); 
	}
	inline String& operator = (const char* szString) 
	{ 
		this->Set(szString); return (*this); 
	}
	inline String& operator + (const String& Other) 
	{ 
		this->Append(Other); return (*this); 
	}
	inline String& operator + (const char* szString) 
	{ 
		this->Append(szString); return (*this); 
	}
	inline const bool operator == (const String& Other) 
	{ 
		return this->Compare(Other); 
	}
	inline const bool operator == (const char* szString) 
	{ 
		return this->Compare(szString); 
	}
	inline operator const char* () 
	{ 
		return this->GetCString(); 
	}

protected:
	void* m_pString;
	TYPE m_Type;
};

};

#endif // STRING_H